//
//  AppDelegate.m
//  MouseFun
//
//  Created by Daniel Prows on 1/20/15.
//
//

#import "AppDelegate.h"
#import "MultiTouch.h"

@interface AppDelegate ()

@property (nonatomic, strong) NSStatusItem* statusBar;
@property (nonatomic, strong) NSMenu *menuItem;

@end

@implementation AppDelegate

const int64_t device_id = 0x400000015dd4d84;
static BOOL shouldTransformClick = NO;
static CFMachPortRef eventTap;
static MTDeviceRef device;

-(void)awakeFromNib
{
	[super awakeFromNib];
	self.menuItem = [[NSMenu alloc] initWithTitle:@""];
	self.statusBar = [[NSStatusBar systemStatusBar] statusItemWithLength:NSVariableStatusItemLength];
	[self.statusBar setTitle:@"RCR"];
	[self.statusBar setHighlightMode:YES];
	[self.statusBar setMenu:self.menuItem];
	
	[self startMonitoring];
}

-(void)startMonitoring
{
	CFMachPortRef eventTap = CGEventTapCreate(kCGHIDEventTap, kCGHeadInsertEventTap, kCGEventTapOptionDefault, kCGEventMaskForAllEvents, handleCGEvent, (__bridge void *)(self));
	CFRunLoopSourceRef runLoopSource = CFMachPortCreateRunLoopSource(kCFAllocatorDefault, eventTap, 0);
	CFRunLoopAddSource(CFRunLoopGetMain(), runLoopSource, kCFRunLoopCommonModes);
	CGEventTapEnable(eventTap, true);
	
	device = MTDeviceCreateFromDeviceID(device_id);
	MTRegisterContactFrameCallback(device, touchCallback);
	MTDeviceStart(device, 0);
	
	[self.menuItem removeAllItems];
	[self.menuItem addItemWithTitle:@"Disable" action:@selector(stopMonitoring) keyEquivalent:@""];
	
	[self.menuItem addItem:[NSMenuItem separatorItem]];
	[self.menuItem addItemWithTitle:@"Quit" action:@selector(terminate:) keyEquivalent:@""];
}

-(void)stopMonitoring
{
	CGEventTapEnable(eventTap, NO);
	eventTap = nil;
	
	MTDeviceStop(device);
	device = nil;
	
	[self.menuItem removeAllItems];
	[self.menuItem addItemWithTitle:@"Enable" action:@selector(startMonitoring) keyEquivalent:@""];
	[self.menuItem addItem:[NSMenuItem separatorItem]];
	[self.menuItem addItemWithTitle:@"Quit" action:@selector(terminate:) keyEquivalent:@""];
}

void touchCallback(MTDeviceRef device, MTTouch touches[], size_t numTouches, double timestamp, size_t frame)
{
	shouldTransformClick = NO;
	if(numTouches == 1)
	{
		MTTouch* touch = &touches[0];
		float x = touch->normalizedVector.position.x;
		if(x >= 0.5)
		{
			shouldTransformClick = YES;
		}
	}
}

CGEventRef handleCGEvent(CGEventTapProxy proxy, CGEventType type, CGEventRef eventRef, void *refcon)
{
	if(shouldTransformClick == NO)
	{
		return eventRef;
	}
	if(type != kCGEventLeftMouseUp && type != kCGEventLeftMouseDown)
	{
		return eventRef;
	}
	NSEvent* event = [NSEvent eventWithCGEvent:eventRef];
	NSLog(@"%@", event);
	if(!event || event.deviceID != device_id)
	{
		return eventRef;
	}
	CGEventSourceRef sourceRef = CGEventCreateSourceFromEvent(eventRef);
	CGPoint point = CGEventGetLocation(eventRef);
	CGEventRef newRef;
	if(type == kCGEventLeftMouseDown)
	{
		newRef = CGEventCreateMouseEvent(sourceRef, kCGEventRightMouseDown, point, kCGMouseButtonRight);
	}
	else
	{
		newRef = CGEventCreateMouseEvent(sourceRef, kCGEventRightMouseUp, point, kCGMouseButtonRight);
	}
	
	return newRef;
}

@end